@@ -43,6 +43,7 @@ BEGIN {
       load_ghc_database
       own_cabal_prerequisites
       hashed_id_to_virtual_installable
+      config_to_package_id
       clean_recipe
       make_setup_recipe
       configure_recipe
@@ -57,6 +58,7 @@ use Carp qw(croak);
 use Const::Fast;
 use Date::Parse qw(str2time);
 use Debian::Debhelper::Dh_Lib qw(doit);
+use File::Temp;
 use IPC::Run3;
 use List::SomeUtils qw(uniq any first_value);
 use Path::Tiny;
@@ -345,11 +347,17 @@ sub ghc_pkg_command {
     my () = @_;
 
     my $inplace_ghc_pkg = 'inplace/bin/ghc-pkg';
+    my $stage2_ghc_pkg = '_build/stage1/bin/ghc-pkg';
 
     # building ghc; use the new ghc-pkg
+    # (this is the old location when using the make build system)
     return $inplace_ghc_pkg
       if -x $inplace_ghc_pkg;
 
+    # (this is the new location when using the Hadrian build system)
+    return $stage2_ghc_pkg
+      if -x $stage2_ghc_pkg;
+
     return 'ghc-pkg';
 }
 
@@ -363,9 +371,16 @@ sub own_cabal_prerequisites {
     croak encode_utf8('No Setup.hs executable named.')
       unless length $ENV{DEB_SETUP_BIN_NAME};
 
-    my $output = run($ENV{DEB_SETUP_BIN_NAME},
+    my $tmp = File::Temp->new();
+    my %options = (
+        stdout => $tmp->filename,
+        update_env => { LC_ALL => 'C.UTF-8'},
+    );
+    doit(\%options, $ENV{DEB_SETUP_BIN_NAME},
         'register', "--builddir=dist-$compiler",
         qw{--gen-pkg-config --verbose=verbose+nowrap});
+    my $output = do { local $/; <$tmp> };
+    say $output if !$ENV{DH_QUIET};
 
     my @hashed_ids;
     if ($output
@@ -467,6 +482,24 @@ sub hashed_id_to_virtual_installable {
     return $virtual;
 }
 
+=item config_to_package_id
+
+=cut
+
+sub config_to_package_id {
+    my ($config, $ghc_pkg, $package_db) = @_;
+
+    my $name = path($config)->basename(qr { [.]conf $}x);
+
+    $name =~ m/^([^ ]*?)-(\d[\d.]*)(-\S+)?$/;
+    $name = "$1-$2";
+
+    my $package_id = run($ghc_pkg, '--package-db', $package_db,
+        qw{--simple-output field}, $name, 'id');
+
+    return $package_id;
+}
+
 =item clean_recipe
 
 =cut
@@ -480,7 +513,7 @@ sub clean_recipe {
     run($ENV{DEB_SETUP_BIN_NAME}, qw{clean})
       if -x $ENV{DEB_SETUP_BIN_NAME};
 
-    run(qw{rm -rf dist dist-ghc dist-ghcjs dist-hugs});
+    run(qw{rm -rf dist dist-ghc dist-ghcjs dist-hugs debian/tmp-setup-hs});
     run(qw{rm -f}, $ENV{DEB_SETUP_BIN_NAME});
     run(qw{rm -f Setup.hi Setup.ho Setup.o});
     run(qw{rm -f}, glob('.*config*'));
